# Leaky Bucket Pattern

Das Leaky Bucket Pattern ist ein Verfahren das zur Verkehrssteuerung ("Traffic Shapping") in Kommunikationssystemen eingesetzt wird.
Es dient zur Begrenzung der Datenrate in einer Kommunikationsverbindung. Die Metapher hinter dem *Leaky Bucket Pattern* ist ein löchriger Eimer (Bucket), 
mit einer gewissen Rate das eingefüllte Wasser (steht für die  übertragenen Datenpakete) verliert. In den Eimer wird in unregelmässigen Abständen Wasser nachgefüllt.
- Ist der Eimer leergelaufen, so so fliesst kein Wasser heraus/keine Daten übertragen
- Ist Wasser im Eimer so läuft er mit einer konstanten Rate aus (Datenpakete werden mit konstanter Rate übertragen)
- Wird zu viel Wasser nachgefüllt, so dass der Eimer überläuft, so geht das überschüssige Wasser verloren (überschüssige Datenpakete werden verworfen).

In folgender Aufgabe verwenden wir das Leaky Bucket Pattern in einem etwas einfacheren Kontext: Ein Datenstrom von ankommenden Datenpaketen (z.B. einer kontinuierlichen Videoübertragung)
soll auf Fehler geprüft werden. Überschreitet die Rate an fehlerhaften Paketen einen Grenzwert von $$ n $$ Fehler/Sekunde, soll ein Fehler geworfen werden.

### Teil 1: Infrastruktur zur Fehlerübertragung

In der Vorlage finden Sie zwei Schnittstellen:
 1. Das ```java interface LeakyBucketPatternInterface``` stellt zwei Methoden zur Verfügung
    - ```java public void setErrorRate(float errorRatePerSec);``` definiert die zulässige Fehlerrate. Vor dem Aufruf ist die zulässige Fehlerraten 0.0. 
    - ```java public void receivePacket(PotentiallyFaultyPacket potentiallyFaultyPacket) throws TooManyErrorsException;;``` definiert eine Methode, die potentiell fehlerhafte Pakete empfängt,
      diese auf Fehler prüft, und falls das Paket fehlerhaft ist, prüft ob in der letzten Sekunde empfangenen Pakete die Fehlerrate  überschritten haben. Falls ja, wird die Ausnahme ```java TooManyErrorsException``` geworfen.
 2. Das ```java interface PotentiallyFaultyPacket``` stellt (als Erweiterung von ```java interface Packet```) zwei Methoden zur Verfügung
   - ```java public byte[] payLoad();``` stellt die übertragenen Daten zur Verfügung. (Diese Methode spielt im folgenden keine Rolle).
   - ```java public boolean isFaulty();``` prüft, ob das übertragene Datenpaket fehlerhaft ist.

```plantuml 
@startuml

skinparam ArrowFontSize 16
skinparam shadowing false
skinparam class {
    FontSize 20
    AttributeFontSize 16
	BorderColor black
	ArrowColor black
}

class PacketChecker {
}

interface LeakyBucketPatternInterface implements PotentiallyFaultyPacket {
  +setErrorRate(float errorRatePerSec)
  +receivePacket(PotentiallyFaultyPacket potentiallyFaultyPacket)
}

interface PotentiallyFaultyPacket {
  +byte[] payLoad()
  +boolean isFaulty()

}

PacketChecker -right-> LeakyBucketPatternInterface  
LeakyBucketPatternInterface .up.> LeakyBucketPatternInterface
@enduml
```

1. [task][Implementieren Sie die Klasse PacketChecker](testClass[PacketChecker],testMethods[PacketChecker],testConstructors[PacketChecker],testNoProblem, testFaulty1, testAccurracy1, testAccurracy2, testAccurracy3, testAccurracy4)
Implementieren Sie die Klasse PacketChecker (im gleichen Paket wie LeakyBucketPatternInterface) und implementieren Sie die Schnittstelle ```java LeakyBucketPatternInterface```.


### Part 2: Testüberdeckung 

2. [task][Implementieren Sie die Klasse PacketChecker](testCoverage)

Schreiben Sie eine Testklasse für PacketChecker unter Verwendung der JUnit4 Infrastruktur.
Diese soll mindestens 75% Testüberdeckung der Klasse PacketChecker sicherstellen.

Ergänzen Sie dazu die Testklasse  at.ac.uibk.qe.artemis.leaky.tests.LeakyBehaviorTest um geeignete Tests. Alternativ können Sie gerne auch  die Testsuite-Klasse at.ac.uibk.qe.artemis.leaky.tests.LeakyTestSuite entsprechend für eigene Tests anpassen.

Bitte beachten Sie, dass Sie (sofern Sie eine eigene Entwicklungsumgebung (außer eclipse) verwenden), ggf. selbst die JUnit-Bibliothek einbinden müssen.